"""
Created on 29 augustus 2019

@author: Tonneau Stephane
@author: Stijn Caerts
@author: Dirk Daems
"""

import tempfile
import json
import logging
import subprocess
from shapely import geometry, wkt
from shapely.validation import explain_validity
from itertools import groupby

LOGGER = logging.getLogger(__name__)


class EsaPolygon(object):
    """ Class that implements polygon support.
    An EsaPolygon is either a Polygon or MultiPolygon (by wrapping over the date line).
    Coordinates are stored in longitude-latitude pairs and in counter-clockwise orientation, according to the GeoJSON convention.
    """

    def __init__(self):
        """
        Initialisator of the class
        """
        self.type = None
        self.geometry = None
        self.bbox_west = None
        self.bbox_south = None
        self.bbox_east = None
        self.bbox_north = None

    def from_posList(self, pos_list, is_lon_lat=True, simplify=True, datelineoffset=10, pole_wrap=False, fix_east_180=False):
        """ Get an EsaPolygon object from a position list.
        :param pos_list: space-separated list of coordinates
        :type pos_list: str
        :param is_lon_lat: indicates whether the list is ordered lon-lat or lat-lon
        :type is_lon_lat: bool
        :param simplify: indicates if the polygon needs to be simplified
        :type simplify: bool
        :param datelineoffset: maximum offset for dateline wrapping to occur
        :type datelineoffset: int
        :param pole_wrap: wrap pole
        :type pole_wrap: bool
        :param fix_east_180: fix longitude coordinates >179.99 to 180 (GDD-2154)
        :type fix_east_180: bool
        """
        # string to list of coordinates
        coordinates = filter(lambda x: len(x) > 0, pos_list.replace("\r", "").replace("\n", "").split(" "))
        coordinates = list(map(float, coordinates))

        # split coordinate list in separate lists for lon and lat coordinates
        if is_lon_lat:
            lon_list, lat_list = coordinates[0::2], coordinates[1::2]
        else:
            lat_list, lon_list = coordinates[0::2], coordinates[1::2]

        if fix_east_180:
            lon_list = list(map(lambda x: 180.0 if x > 179.99 else x, lon_list))

        poly = geometry.Polygon([[lon, lat] for lat,lon in zip(lat_list, lon_list)])
        return self.from_polygon(poly, simplify, datelineoffset, pole_wrap)

    def from_wkt(self, wkt_geometry, simplify=True, datelineoffset=10, pole_wrap=False):
        """ Get an EsaPolygon object from a WKT input
        :param wkt_geometry: geometry in WKT format
        :type wkt_geometry: str
        :param simplify: indicates if the polygon should be simplified
        :type simplify: bool
        :param datelineoffset: maximum offset for dateline wrapping to occur
        :type datelineoffset: int
        """
        poly = wkt.loads(wkt_geometry)
        return self.from_polygon(poly, simplify, datelineoffset, pole_wrap)

    def from_geojson(self, geojson, simplify=True, datelineoffset=10, pole_wrap=False):
        """ Get an EsaPolygon object from a GeoJSON input
        :param geojson: geometry in GeoJSON format
        :type geojson: dict
        :param simplify: indicates if the polygon should be simplified
        :type simplify: bool
        :param datelineoffset: maximum offset for dateline wrapping to occur
        :type datelineoffset: int
        """

        poly = geometry.shape(geojson)
        return self.from_polygon(poly, simplify, datelineoffset, pole_wrap)

    def from_polygon(self, poly, simplify=True, datelineoffset=10, pole_wrap=False):
        # simplify Polygon and perform dateline wrapping
        # use the RFC7946 layer creation option to ensure CCW orientation
        # https://gdal.org/drivers/vector/geojson.html#rfc-7946-write-support
        with tempfile.TemporaryDirectory() as tmp_dir:
            input_filename = "{}/in.geojson".format(tmp_dir)
            output_filename = "{}/out.geojson".format(tmp_dir)
            with open(input_filename, 'w') as input_file:
                input_file.write(json.dumps(geometry.mapping(poly)))

            def ogr2ogr(input_filename, output_filename, simplify, datelineoffset):
                # use ogr2ogr command iso python wrapper as GDAL version on dataflow instances is not upgraded yet (GDD-456)
                command = ["/usr/bin/ogr2ogr", "-f", "GeoJSON", "-wrapdateline", "-datelineoffset", str(datelineoffset),
                           "-lco", "RFC7946=YES", "-lco", "WRITE_BBOX=YES"]
                if simplify:
                    command.append("-simplify")
                    command.append("0.005")
                command.append(output_filename)
                command.append(input_filename)
                subprocess.run(command, check=True)

            if pole_wrap and not poly.is_valid and "Self-intersection" in explain_validity(poly):
                # try stereographic pole reprojection
                def reproject_polar(input_filename, output_filename, srs):
                    tmp_filename = "{}/tmp.geojson".format(tmp_dir)
                    command = ["/usr/bin/ogr2ogr", "-f", "GeoJSON", "-t_srs", srs, "-s_srs", "EPSG:4326",
                               "-lco", "WRITE_BBOX=YES", tmp_filename, input_filename]
                    subprocess.run(command, check=True)
                    command = ["/usr/bin/ogr2ogr", "-f", "GeoJSON", "-s_srs", srs, "-t_srs", "EPSG:4326",
                               "-segmentize", "250000", "-wrapdateline", "-datelineoffset", str(datelineoffset),
                               "-lco", "WRITE_BBOX=YES", "-lco", "RFC7946=YES", output_filename, tmp_filename]
                    subprocess.run(command, check=True)

                _, miny, _, maxy = poly.bounds
                if miny < -80:
                    LOGGER.debug("Reprojecting to Antarctic Polar Stereographic (EPSG:3031)")
                    reproject_polar(input_filename, output_filename, "EPSG:3031")
                elif maxy > 80:
                    LOGGER.debug("Reprojecting to Arctic Polar Stereographic (EPSG:3995)")
                    reproject_polar(input_filename, output_filename, "EPSG:3995")
                else:
                    # not close enough to poles, fall back to normal case
                    ogr2ogr(input_filename, output_filename, simplify, datelineoffset)
            else:
                ogr2ogr(input_filename, output_filename, simplify, datelineoffset)

            # load simplified geometry from output
            with open(output_filename) as f:
                jf = json.load(f)
                # get geometry
                self.geometry = jf["features"][0]["geometry"]
                # remove the duplicate consecutive coordinates as ElasticSearch doesn't like this
                self.geometry["coordinates"][0] = [k for k, _ in
                                                   groupby(jf["features"][0]["geometry"]["coordinates"][0])]
                # get bounding box
                bbox = jf["features"][0]["bbox"]
                self.bbox_west = bbox[0]
                self.bbox_south = bbox[1]
                self.bbox_east = bbox[2]
                self.bbox_north = bbox[3]

        return self

    def as_geojson_string(self):
        """ Get a GeoJSON string representation of the polygon
        :returns: a string containing the GeoJSON representation
        :rtype: str
        """
        return json.dumps(self.geometry)

    def get_bbox(self):
        """ Get the bounding box of the polygon. The order of the values is [west, south, east, north].
        :returns: the bounding box
        """
        return [self.bbox_west, self.bbox_south, self.bbox_east, self.bbox_north]
