import logging
import re
import time
import os
import pyproj

from oscars_mgmnt.geometry.polygon import EsaPolygon
from oscars_mgmnt.parser.base import BaseParser, SpecificInfo
from oscars_mgmnt.utils.es_rest_support import ElasticRestSupport
from oscars_mgmnt.file_object import ObjectInfo, FileInfo

LOGGER = logging.getLogger(__name__)
TRANSFORM_4326_TO_3857 = pyproj.Transformer.from_crs("EPSG:4326", "EPSG:3857", always_xy=True)


class ElasticsearchParser(BaseParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        BaseParser.__init__(self, template)
        self.es_rest_support = ElasticRestSupport(hosts=es_source_host)
        self.es_index = es_source_index

        # regex used to retrieve the product identifier:
        #   retrieve product identifier based on folder name
        #   look for a string that contains date information, like 20200727T173023
        self.regex_identifier = re.compile(r".*/(\w*_\d{8}T\d{6}_\w*)(?:($)|(/.*))")

    def init_dict(self):
        self.dict_info = {
            "title": "title",
            "geometry": "geometry",
            "footprint": "properties.footprint",
            "orbitnumber": "properties.orbitnumber",
            "orbitdirection": "properties.orbitdirection",
            "relativeorbitnumber": "properties.relativeorbitnumber",
            "beginposition": "properties.beginposition",
            "endposition": "properties.endposition",
            "nominaldatetime": "properties.beginposition",
            "updatetime": "properties.processing_status.processing_end",
            "producttype": "properties.producttype",
            "sensormode": "properties.sensoroperationalmode",
            "processingmode": "properties.processingmode",
            "timeliness": "properties.timeliness",
            "inputs": "inputs"
        }

    def convert(self, product_file_info):
        """ retrieves the product metadata from Elasticsearch based on the product identifier
        :param product_file_info: the description of the files associated to a product
        :type product_file_info: ObjectInfo
        """

        # retrieve product identifier using regex pattern
        match = self.regex_identifier.search(product_file_info.get_product_path())

        if match and len(match.groups()) > 0:
            product_identifier = match.group(1)
        else:
            raise Exception("Could not derive the product identifier from the product path {}"
                              .format(product_file_info.get_product_path()))

        # get document from Elasticsearch
        metadata = self.get_metadata(product_identifier)

        # Initialize values dict
        values = {}
        values['id'] = product_identifier

        # Loop over all the property names and add them to the values if they are present
        for k, v in self.dict_info.items():
            tmp = self.extract_value(metadata, v)
            if tmp is not None:
                values[k] = tmp

        # Generate other properties based on extracted values
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())

        specific_infos = self.splitted_treatment(product_file_info, values)
        self.fix_id(values)

        # Generation of the output
        results = {}

        if self.jinja_template is None:
            self.with_template(self.template)

        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def get_metadata(self, product_identifier):
        """ Get the product metadata from Elasticsearch
        :param product_identifier: product identifier
        :type product_identifier: str
        """
        return self.es_rest_support.get(self.es_index, product_identifier)

    def extract_value(self, metadata, path):
        """
        Extract a property, given by the path, from the metadata
        :param metadata: the metadata
        :type metadata: dict
        :param path: path of the of the property in the metadata dict
        :type path: str
        """
        path = path.split(".")
        tmp = metadata
        for key in path:
            if key in tmp:
                tmp = tmp[key]
            else:
                return None
        return tmp

    def fix_id(self, values):
        """ Fix the ID of the product by prepending the parent identifier
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """
        values['id'] = values['parentidentifier'] + ":" + values['title']

    def fix_geometry(self, values, simplify=True):
        """ Simplify the geometry and generate the bbox
        :param values: dictionary containing the values, must include the geometry
        :type values: dict
        """
        if "geometry" in values:
            polygon = EsaPolygon().from_geojson(values['geometry'], simplify=simplify)
        else:
            polygon = EsaPolygon().from_wkt(values['footprint'], simplify=simplify)
        values['geometry'] = polygon.as_geojson_string()
        values['westbbox'], values['southbbox'], values['eastbbox'], values['northbbox'] = polygon.get_bbox()

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        uid = values['title']
        if "_V" in uid:
            values['productVersion'] = uid[uid.rfind("_V")+1:]

        self.fix_products(product_file_info)

        return uid

    def fix_products(self, product_file_info):
        """" Helper method that moves files in the product file info to the correct section
        :param product_file_info: product file info
        :type product_file_info: ObjectInfo
        """
        raise NotImplementedError("Implement this method in a subclass.")

    def add_wms_quicklook(self, wms_layer: str, product_file_info: ObjectInfo, values: dict):
        west, south = TRANSFORM_4326_TO_3857.transform(values['westbbox'], values['southbbox'])
        east, north = TRANSFORM_4326_TO_3857.transform(values['eastbbox'], values['northbbox'])

        # in WMS 1.3.0, the coordinate order is lat-lon
        wms_link = f"https://services.terrascope.be/wms/v2" \
                   "?SERVICE=WMS" \
                   "&REQUEST=getMap" \
                   "&VERSION=1.3.0" \
                   "&CRS=EPSG:3857" \
                   "&SRS=EPSG:3857" \
                   f"&LAYERS={wms_layer}" \
                   f"&TIME={values['endposition'][:10]}" \
                   f"&BBOX={west},{south},{east},{north}" \
                   "&WIDTH=80" \
                   "&HEIGHT=80" \
                   "&FORMAT=image/png" \
                   "&TRANSPARENT=true"

        wms_quicklook = FileInfo(wms_link, file_type="image/png", file_title="WMS")
        product_file_info.previews.append(wms_quicklook)

    @staticmethod
    def format_uid_date(uid_date):
        return '{}-{}-{}T{}:{}:{}Z'.format(uid_date[0:4], uid_date[4:6], uid_date[6:8],
                                           uid_date[9:11], uid_date[11:13], uid_date[13:15])


class Sentinel1_ES_Parser(ElasticsearchParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S1.template' if template is None else template, es_source_host, es_source_index)

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "polarisationmode": "properties.polarisationmode",
                "srs": "properties.srs"
            }
        )

    def specific_treatment(self, product_file_info, values):
        uid = super().specific_treatment(product_file_info, values)
        self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]

        if "SIGMA0" in values["producttype"]:
            values["producttype"] = "SIGMA0"
            values['parentidentifier'] = "urn:eop:VITO:CGS_S1_GRD_SIGMA0_L1"
        elif "COHERENCE" in values["producttype"]:
            values["producttype"] = "COHERENCE"
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S1_SLC_COHERENCE_V1"
            values['nominaldatetime'] = self.format_uid_date(uid.split("_")[4])

            # don't include platformserialid for coherence products
            values.pop('platformserialid', None)
        elif values["producttype"] in ["GRD", "SLC"]:
            values['parentidentifier'] = f"urn:eop:VITO:CGS_S1_{values['producttype']}_L1"
            values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"

        if "srs" in values:
            values['referenceSystemIdentifier'] = values['srs']
            values['conformsto'] = f"https://www.opengis.net/def/crs/EPSG/0/{values['srs'].split(':')[1]}"

        polarisations = {
            "SH": {"mode": "S", "channels": "HH"},
            "SV": {"mode": "S", "channels": "VV"},
            "DH": {"mode": "D", "channels": "HH, HV"},
            "DV": {"mode": "D", "channels": "VV, VH"},
            "HH": {"mode": "D", "channels": "HH"},
            "HV": {"mode": "D", "channels": "HV"},
            "VV": {"mode": "D", "channels": "VV"},
            "VH": {"mode": "D", "channels": "VH"}
        }

        polarisation = [v for v in polarisations.values() if all(x in v['channels'] for x in values['polarisationmode'].split())].pop()

        values["polarisationmode"] = polarisation["mode"]
        values["polarisationchannels"] = polarisation["channels"]

        wms_layer = "CGS_S1_COHERENCE" if "COHERENCE" in values["producttype"] else "CGS_S1_GRD_SIGMA0"
        self.add_wms_quicklook(wms_layer, product_file_info, values)

        return uid

    def fix_products(self, fileobject):
        grd_pattern = re.compile(r".*S1._.._GRDH_.*.zip")
        slc_pattern = re.compile(r".*S1._.._SLC__.*.zip")
        sigma0_pattern = re.compile(r".*S1._.._GRDH_SIGMA0_.*_V\d{3}_(angle|VV|VH).tif")
        coherence_pattern = re.compile(r".*S1._S1._Coherence_.*_V\d{3}_(VV|VH).tif")

        for info in list(fileobject.datas):
            if sigma0_pattern.match(info.name):
                info.title = sigma0_pattern.search(info.name).group(1)
            elif coherence_pattern.match(info.name):
                info.title = coherence_pattern.search(info.name).group(1)
            elif grd_pattern.match(info.name):
                info.title = "GRD"
            elif slc_pattern.match(info.name):
                info.title = "SLC"


class Sentinel2_ES_Parser(ElasticsearchParser):

    pattern_terrascope = re.compile(r"S2._\d{8}T\d{6}_[A-Z0-9]{5}_([A-Z]*)_V(\d{3})")
    terrascope_land_producttypes =  ["CCC", "CWC", "FAPAR", "FCOVER", "LAI", "NDVI", "TOC"]
    terrascope_water_producttypes = ["CHL", "RHOW", "SPM", "TUR"]
    wms_layer_mapping = {
        # Terrascope land
        "CCC": "CGS_S2_RADIOMETRY",
        "CWC": "CGS_S2_RADIOMETRY",
        "FAPAR": "CGS_S2_FAPAR",
        "FCOVER": "CGS_S2_FCOVER",
        "LAI": "CGS_S2_LAI",
        "NDVI": "CGS_S2_NDVI",
        "TOC": "CGS_S2_RADIOMETRY",
        # Terrascope water
        "CHL": "TERRASCOPE_S2_CHL_V1",
        "RHOW": "TERRASCOPE_S2_RHOW_V1",
        "SPM": "TERRASCOPE_S2_SPM_V1",
        "TUR": "TERRASCOPE_S2_TUR_V1"
    }

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S2.template' if template is None else template, es_source_host, es_source_index)

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "cloudcover": "properties.cloudcoverpercentage",
                "tileid": "properties.tileid",
                "processingbaseline": "properties.processingbaseline"
            }
        )

    def get_metadata(self, product_identifier):
        # replace product type with CGS, eg. S2A_20200323T030541_49QCE_TOC_V200 -> S2A_20200323T030541_49QCE_CGS_V200
        if self.pattern_terrascope.match(product_identifier):
            producttype = self.pattern_terrascope.search(product_identifier).group(1)
            if producttype in self.terrascope_land_producttypes:
                doc_id = product_identifier.replace(producttype, "CGS")
                return super().get_metadata(doc_id)
            elif producttype in self.terrascope_water_producttypes:
                doc_id = product_identifier.replace(producttype, "WATER")
                return super().get_metadata(doc_id)
        else:
            # fall back to original identifier
            return super().get_metadata(product_identifier)

    def specific_treatment(self, product_file_info, values):
        uid = super().specific_treatment(product_file_info, values)
        # fix geometry of L1C products
        # these may contain duplicate consecutive coordinates, which causes ES ingest to fail
        simplify = True if "_MSIL1C_" in uid else False
        self.fix_geometry(values, simplify=simplify)

        values['platformserialid'] = uid.split("_")[0]

        if self.pattern_terrascope.match(values['id']):
            producttype = self.pattern_terrascope.search(values['id']).group(1)
            values['producttype'] = producttype
            resolution_dir_regex = re.compile(r".*/(\d+)M")
            resolution_file_regex = re.compile(rf".*_{producttype}.*_(\d+)M_.*")

            if resolution_dir_regex.match(product_file_info.get_product_path()):
                # include resolution in product identifier, eg. FAPAR
                resolution = resolution_dir_regex.search(product_file_info.get_product_path()).group(1)
                values['resolution'] = int(resolution)
                values['id'] = f"{values['id'].rsplit('_', 1)[0]}_{resolution}M_{values['id'].rsplit('_', 1)[1]}"
            elif all(resolution_file_regex.match(file.name) for file in product_file_info.datas):
                resolutions = {resolution_file_regex.search(file.name).group(1) for file in product_file_info.datas}
                if len(resolutions) == 1:
                    # only one resolution, add resolution to product identifier, eg. CCC
                    resolution = resolutions.pop()
                    values['resolution'] = int(resolution)
                    values['id'] = f"{values['id'].rsplit('_', 1)[0]}_{resolution}M_{values['id'].rsplit('_', 1)[1]}"

            values['title'] = values['id']
            uid = values['id']
            values['parentidentifier'] = f"urn:eop:VITO:TERRASCOPE_S2_{values['producttype']}_{values['productVersion'][:2]}"
            if 'tileid' not in values:
                values['tileid'] = uid.split("_")[2]

            wms_layer = self.wms_layer_mapping[values['producttype']]
            self.add_wms_quicklook(wms_layer, product_file_info, values)
        else:
            # non-V200 product
            # change producttype from eg. S2MSI1C to L1C
            values['producttype'] = f"L{values['producttype'][-2:]}"
            values['parentidentifier'] = f"urn:eop:VITO:CGS_S2_{values['producttype']}"

            if "_OPER_" in uid and 'productVersion' in values:
                # for these kind of products a faulty processor version is extracted from the identifier
                values.pop('productVersion')

        return uid

    def fix_products(self, fileobject):
        """ helper method that moves files present in "datas" to "related".
        :param fileobject: the files associated to a product
        :type fileobject: a FileObject
        """

        # the ?: in front makes it non-capturing (so it doesn't introduce and additional group)
        shadow_pattern = re.compile(r'.*((?:SHADOWMASK_\d{2}M)|RAA_\d{2}M|VZA_\d{2}M|SZA_\d{2}M|WVP_\d{2}M|AOT_\d{2}M).*')
        cloud_pattern = re.compile(r'.*(CLOUDMASK_\d{2}M).*')
        scene_pattern = re.compile(r'.*(SCENECLASSIFICATION_\d{2}M).*')
        pixel_pattern = re.compile(r'.*(PIXELCLASSIFICATION_(\d{2}M)?).*')
        biopar_pattern = re.compile(r'.*((?:FAPAR|NDVI|FCOVER|LAI|CCC|CWC)_\d{2}M).*')
        water_param_pattern = re.compile(r'.*((?:CHL|SPM|TUR|CHL_SOURCE))_\d{2}M.*')
        radiometry_pattern = re.compile(r'.*((?:TOC-B\d\d_\d{2}M|TOC-B8A_20M)).*')
        water_rhow_pattern = re.compile(r'.*RHOW-((?:B\d\d|B8A))_\d{2}M.*')
        L1C_pattern = re.compile(r'.*_((?:TCI|B\d{2}|B8A))\..*')
        SAFE_pattern = re.compile(r'.*\.SAFE/(.*)')

        for info in list(fileobject.datas):

            if shadow_pattern.match(info.name):
                info.title = shadow_pattern.search(info.name).group(1)
                info.category = "QUALITY"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif cloud_pattern.match(info.name):
                info.title = cloud_pattern.search(info.name).group(1)
                info.category = "CLOUD"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif scene_pattern.match(info.name):
                info.title = scene_pattern.search(info.name).group(1)
                info.category = "QUALITY"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif pixel_pattern.match(info.name):
                info.title = pixel_pattern.search(info.name).group(1)
                info.category = "QUALITY"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif biopar_pattern.match(info.name):
                info.title = biopar_pattern.search(info.name).group(1)
            elif radiometry_pattern.match(info.name):
                info.title = radiometry_pattern.search(info.name).group(1)
            elif water_param_pattern.match(info.name):
                info.title = water_param_pattern.search(info.name).group(1)
            elif water_rhow_pattern.match(info.name):
                info.title = water_rhow_pattern.search(info.name).group(1)
            elif L1C_pattern.match(info.name):
                info.title = L1C_pattern.search(info.name).group(1)

        for alternate_info in list(fileobject.metadatas):
            # move XML files inside the SAFE folder to the related section
            if SAFE_pattern.match(alternate_info.name):
                fileobject.metadatas.remove(alternate_info)
                fileobject.related.append(alternate_info)

        for related_info in list(fileobject.related):
            if SAFE_pattern.match(related_info.name):
                related_info.title = SAFE_pattern.search(related_info.name).group(1)
                if related_info.type is None and related_info.name.endswith(".gml"):
                    related_info.type = "application/gml+xml"
                if related_info.category is None and "/QI_DATA/" in related_info.name:
                    related_info.category = "QUALITY"


class CLMS_HRVPP_ES_Parser(ElasticsearchParser):

    # Per theme, we define 
    #     a dataset (collection) identifier, with substitutions for 
    #       projection (EPSG code), resolution, year(s) and short version
    #     a full name for the theme
    dict_theme = {
        'VI': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-vi_p_%s_%s',
            'name': 'Vegetation Indices'
        },
        'ST': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-st_p_%s_%s',
            'name': 'Seasonal Trajectories'
        },
        'VPP': {
            'dataset_identifier': 'copernicus_r_%s_%s_m_hrvpp-vpp_p_%s_%s',
            'name': 'Vegetation Phenology and Productivity parameters'
        }
    }

    # regex to recognize the region name as a S2 tile (syntax TddXXX) 
    s2tile_regex=re.compile(r"^T\d{2}")

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CLMS_HRVPP.template' if template is None else template, es_source_host, es_source_index)
        # regex used to retrieve the product identifier
        self.regex_identifier = re.compile(r".*/([\w-]*)(?:\.(?:tif|zip)$)")

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                # "cloudcover": "properties.cloudcoverpercentage",
                "l1cidentifier": "properties.l1cidentifier"
            }
        )

    def specific_treatment(self, product_file_info, values):
        # Skip the super() method intentionally

        # File name convention:
        # THEME_TIME_[MISSION]_EXTENT-RESOLUTION_VERSION_MODE_FILE-ID
        # example: VI_20180814T100031_S2A_T32TPN-010m_V100_NDVI.tif
        # See URD for more information

        # product identifier (values['id']) should have the form
        # THEME_TIME_[MISSION]_EXTENT-RESOLUTION_VERSION_MODE
        product_base = values['id']
        filename_elem = product_base.split("_")

        mode = None
        mode_long = None
        mission = 'S2'
        if len(filename_elem)==7:
            (theme,nominaltime,mission,spatial,product_version,mode,fileid)=filename_elem
        elif len(filename_elem)==6:
            (theme,nominaltime,mission,spatial,product_version,fileid)=filename_elem

        # Spatial grid information:
        #   geographic area name, projection (EPSG code) and resolution
        #   e.g. E58N22-03035-010m
        spatial_elem = spatial.split('-')
        if self.s2tile_regex.match(spatial_elem[0]):
            # region is a S2 tile
            region, resolution = spatial_elem

            # don't include the starting T in the tileid
            values['tileid'] = region[1:]

            epsg = 'utm-wgs84'
            # conformsto is retained from the ES metadata (S2 tiling grid -> UTM zone)
        else:
            # ETRS89-LAEA = EPSG:3035 projection

            # country code and EPSG (eg at-03035)
            region, epsg, resolution = spatial_elem

            # region is a 2-char ISO code of a country
            values['tileid'] = region

            epsg=epsg.lstrip('0')
            values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/3035"

        resolution_value = int(resolution[:-1])
        values['resolution'] = resolution_value

        self.fix_products(product_file_info, epsg, values['resolution'])
        self.fix_geometry(values, simplify=False)
        
        # Platform "Sentinel-2" is fixed in the template file.
        # Platform's serial id: filename convention allows  S2A, S2B or S2 (= S2A+S2B)
        mission = mission.upper()
        if mission.upper() == "S2":
            values['platformserialid'] = "S2A, S2B"
        else:
            values['platformserialid'] = mission

        values['producttype'] = fileid.upper()

        # Product type / theme
        theme = theme.upper()
        if theme == 'VPP':
            values['productgroupid'] = mode
        
        # Product version
        #     product_version "Vxyy" like in filename; SDI convention for 'edition' element of ISO XML has xx.yy instead
        #     version_short "vxx_ryy" is used in parent dataset identifier
        #     version_long "version xx revision yy" is used in title
        #  where x = major version, yy = revision / minor version
        product_version=product_version.upper()
        majorversion= int(product_version[1])
        minorversion= product_version[-2:]

        # Parent dataset and collection keep revision = 01
        # to keep V101 and V102 products together
        # version_short = 'v'+ "%02d" % majorversion+'_r'+minorversion
        version_short = 'v'+ "%02d" % majorversion
        version_long = ' '.join(['version', str(majorversion) ,'revision',minorversion])
        if mode:
            mode_long = 'season ' + mode[1]

        # drop the revision in version_long when revision = 00
        if minorversion == '00':
            version_long  = 'version ' + str(majorversion)

        values['productVersion'] = product_version
        year = '2017-now'
        
        # Title: a readable title, composed of
        #     dataset (collection) title, with substitutions for theme's name, year(s), resolution and long version
        #     tile/area name and nominal time (to identify this product within the collection)
        #     layer name (fileid) is included, as it is used in the file info (per COG file) instead
        #     (VPP parameters only) season 1 or season 2

        if mode:
            values['title'] = '%s %s (raster %s) - %s : %s %s %s %s' % ( self.dict_theme[theme]['name'], year, resolution, version_long, fileid.upper(), region, nominaltime, mode_long)
        else:
            values['title'] = '%s %s (raster %s) - %s : %s %s %s' % ( self.dict_theme[theme]['name'], year, resolution, version_long, fileid.upper(), region, nominaltime)

        if theme == 'VI':
            # example identifier for the parent L1C product: S2B_MSIL1C_20200104T101319_N0208_R022_T33UVS_20200104T121239
            # OPTIONAL - relative orbit nr
            values ['relativeorbitnumber'] = int(values['l1cidentifier'].split('_')[4].replace('R',''))

        if epsg == 'utm-wgs84':
            values['parentidentifier'] = self.dict_theme[theme]['dataset_identifier'] % (epsg, str(resolution_value), year, version_short)
        else: # LAEA collection with mixed resolutions:
            values['parentidentifier'] = self.dict_theme[theme]['dataset_identifier'] % (epsg, 'x', year, version_short)

        # the product ID is retained from convert() and updated by fix_id() call         
        uid = values['id']
        
        return uid

    def fix_id(self, values):
        """ Overwritten - No need to prepend the parent identifier for HR-VPP
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """

    def fix_products(self, fileobject, epsg, resolution):
        """ helper method that moves QA files present in "datas" to "related" and updates file info title
        :param fileobject: the files associated to a product
        :param epsg: EPSG code
        :param resolution: image resolution
        :type fileobject: a FileObject
        """

        for info in list(fileobject.datas):
            file_base = os.path.splitext(os.path.basename(info.name))[0]
            filename_elem = file_base.split("_")

            if epsg == 'utm-wgs84':
                info.title = filename_elem[-1].upper()
            else:
                info.title = filename_elem[-1].upper() + '_' + str(resolution) + 'M'
            
            # add human readable name of the fileid (layer) to file info or leave this up to front-end?
            if "QFLAG" in info.title or "QFLAG2" in info.title:
                info.category = "QUALITY"
                # fileobject.datas.remove(info)
                # fileobject.related.append(info)


class Sentinel5P_ES_Parser(ElasticsearchParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S5P.template' if template is None else template, es_source_host, es_source_index)
        self.regex_identifier = re.compile(r".*/(\w*_\d{8}_\w*)(?:($)|(/.*))")

    def specific_treatment(self, product_file_info, values):
        #remove processingmode from id
        old_title= values['title']
        new_title=old_title.replace('_OFFL','')
        new_title=new_title.replace('_NRTI','')
        new_title = new_title.replace('_RPRO','')
        values['title'] = new_title
        values['id'] = new_title

        uid = super().specific_treatment(product_file_info, values)
        self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]
        values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"

        if "NO2_TD" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_NO2_TD_V1"
            #values.pop('platformserialid', None)
        elif "CO_TD" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CO_TD_V1"
            #values.pop('platformserialid', None)
        elif "HCHO_TD" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_HCHO_TD_V1"
            # values.pop('platformserialid', None)
        elif "CH4_TD" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CH4_TD_V1"
            # values.pop('platformserialid', None)
        elif "NO2_TM" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_NO2_TM_V1"
            # values.pop('platformserialid', None)
        elif "CO_TM" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CO_TM_V1"
            # values.pop('platformserialid', None)
        elif "HCHO_TM" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_HCHO_TM_V1"
            # values.pop('platformserialid', None)
        elif "CH4_TM" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CH4_TM_V1"
            # values.pop('platformserialid', None)
        elif "NO2_TY" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_NO2_TY_V1"
            # values.pop('platformserialid', None)
        elif "CO_TY" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CO_TY_V1"
            # values.pop('platformserialid', None)
        elif "HCHO_TY" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_HCHO_TY_V1"
            # values.pop('platformserialid', None)
        elif "CH4_TY" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S5P_L3_CH4_TY_V1"
            # values.pop('platformserialid', None)
        return uid


    def fix_products(self, fileobject):
        #S5P_NO2_TD_20200131_NO2_V100.tif|S5P_NO2_TD_20200131_WEIGHT_V100.tif
        s5p_main_pattern = re.compile(".*S5P_([NO2HC4_TDMY]*)_[0-9]{4,8}_([NO2CH4]{2,6})_V[0-9]{3}.tif")
        s5p_weight_pattern = re.compile(".*S5P_([NO2HC4_TDMY]*)_[0-9]{4,8}_([WEIGHT]{2,6})_V[0-9]{3}.tif")

        for info in list(fileobject.datas):
            if s5p_main_pattern.match(info.name):
                m=s5p_main_pattern.search(info.name)
                info.title = m.group(2)
            if s5p_weight_pattern.match(info.name):
                m=s5p_weight_pattern.search(info.name)
                info.category = "QUALITY"
                info.title = m.group(2)
                fileobject.datas.remove(info)
                fileobject.related.append(info)
class Sentinel3_Parser(ElasticsearchParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        super().__init__('CGS_S3.template' if template is None else template, es_source_host, es_source_index)

    def init_dict(self):
        super().init_dict()
        self.dict_info.update(
            {
                "cogProcessingDate": "properties.processing_status.cog_processing_status.processing_end",
                "pdutileid": "properties.pdutileid",
            }
        )

    def fix_products(self, fileobject):
        for info in list(fileobject.datas):
            name = os.path.basename(info.name)
            name = name.split('.')[0]
            info.title = os.path.basename(name)
            info.href = "file:///" + info.name
            info.conformsTo = "http://www.opengis.net/def/crs/EPSG/0/4326"


    def specific_treatment(self, product_file_info, values):
        uid = super().specific_treatment(product_file_info, values)
        self.fix_geometry(values, simplify=False)

        values['platformserialid'] = uid.split("_")[0]
        values['conformsto'] = "http://www.opengis.net/def/crs/EPSG/0/4326"
        continent = values['pdutileid']
        continent = continent.rstrip('_')
        values['productGroupId'] = continent

        if "SY_2_V10___" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_SY_2_V10_V1"
            values['compositeType'] = "10-DAILY"
            #values.pop('platformserialid', None)
        elif "SY_2_VG1___" in values["producttype"]:
            values['parentidentifier'] = "urn:eop:VITO:TERRASCOPE_S3_SY_2_VG1_V1"
            values['compositeType'] = "DAILY"
            #values.pop('platformserialid', None)

        return uid