from oscars_mgmnt.parser.base import BaseParser
from oscars_mgmnt.file_object import ObjectInfo
from oscars_mgmnt.utils.storage import S3Input


class WorldCoverParser(BaseParser):
    """ Parser used to transform ESA WorldCover filename to GeoJSON."""

    def __init__(self):
        super().__init__(template='WorldCover.template')

    def convert(self, product_file_info):

        import os
        import time

        # Initialize values dict
        values = {}

        # filename without extension
        file_name = os.path.basename(product_file_info.get_product_path()).split('.')[0]
        file_name_parts = file_name.split('_')

        year = file_name_parts[3]
        tile_id = file_name_parts[5]
        type = file_name_parts[6] # Map, S1VVVHratio or S2RGBNIR

        version = file_name_parts[4] # v100 or V200, ...
        major_version = int(version[1])

        if type == 'Map':
            values['producttype'] = 'ESA_WORLDCOVER_10M'
            if isinstance(product_file_info.storage, S3Input):
                parent_identifier_template = 'urn:eop:VITO:ESA_WorldCover_10m_{year}_AWS_V{major_version}'
            else:
                parent_identifier_template = 'urn:eop:VITO:ESA_WorldCover_10m_{year}_V{major_version}'
            parent_identifier = parent_identifier_template.format(year=year, major_version=major_version)
            title = file_name.rsplit('_', 1)[0]
        elif type in {'S1VVVHratio', 'S2RGBNIR', 'SWIR', 'NDVI'}:
            values['producttype'] = f'ESA_WORLDCOVER_{type}_10M'
            parent_identifier = 'urn:eop:VITO:ESA_WorldCover_{type}_10m_{year}_V{major_version}'.format(
                type=type, year=year, major_version=major_version)
            title = file_name
        else:
            raise Exception('Unsupported WorldCover type {0}'.format(type))

        bbox = self.get_bbox(tile_id, type)

        values['id'] = parent_identifier + ':' + title
        values['title'] = title
        values['parentidentifier'] = parent_identifier
        values['conformsto'] = 'https://www.opengis.net/def/crs/EPSG/0/4326'
        values['westbbox'] = bbox[0]
        values['southbbox'] = bbox[1]
        values['eastbbox'] = bbox[2]
        values['northbbox'] = bbox[3]
        values['tileid'] = tile_id
        values['resolution'] = 10

        values['beginposition'] = '{0}-01-01T00:00:00Z'.format(year)
        values['endposition'] = '{0}-12-31T23:59:59Z'.format(year)
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['updatetime'] = time.strftime(
            '%Y-%m-%dT%H:%M:%SZ',
            product_file_info.storage.get_time_modified(product_file_info.get_product_path())
        )

        if self.jinja_template is None:
            self.with_template(self.template)

        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):

        for file_info_data in list(product_file_info.datas):
            if file_info_data.name.endswith('_Map.tif'):
                file_info_data.title = 'ESA_WORLDCOVER_10M_MAP'
            elif file_info_data.name.endswith('_InputQuality.tif'):
                file_info_data.title = 'ESA_WORLDCOVER_10M_INPUTQUALITY'
                file_info_data.category = 'QUALITY'
                product_file_info.datas.remove(file_info_data)
                product_file_info.related.append(file_info_data)
            elif file_info_data.name.endswith('_S2RGBNIR.tif'):
                file_info_data.title = 'S2RGBNIR'
            elif file_info_data.name.endswith('S1VVVHratio.tif'):
                file_info_data.title = 'S1VVVHratio'
            elif file_info_data.name.endswith('SWIR.tif'):
                file_info_data.title = 'SWIR'
            elif file_info_data.name.endswith('NDVI.tif'):
                file_info_data.title = 'NDVI'

        return values['id']

    @staticmethod
    def get_bbox(file_name_part, type):

        import re
        result = re.search('([N|S])(\d\d)([E|W])(\d\d\d)',
                           file_name_part)

        if type == 'Map':
            tile_size_degrees = 3
        else:
            tile_size_degrees = 1

        n_s = result.group(1)
        e_w = result.group(3)
        n_s_degrees = result.group(2)
        e_w_degrees = result.group(4)

        west_bbox = int(e_w_degrees) if e_w == 'E' else int(e_w_degrees) * -1
        east_bbox = west_bbox + tile_size_degrees
        south_bbox = int(n_s_degrees) if n_s == 'N' else int(n_s_degrees) * -1
        north_bbox = south_bbox + tile_size_degrees

        return [west_bbox, south_bbox, east_bbox, north_bbox]
