"""
Created on 30 June 2019

@author: Tonneau Stephane
"""

import re
import copy
from oscars_mgmnt.geometry.tiles import TileInfo
from oscars_mgmnt.geometry.polygon import EsaPolygon
from oscars_mgmnt.parser.base import XMLMetaParser, SpecificInfo
from oscars_mgmnt.file_object import ObjectInfo


class HmaParser(XMLMetaParser):
    '''class that able to parse HMA metadafile to create geojson
    equivalence.'''

    def __init__(self, template=None):
        """ Initialisator of the class
        """
        super().__init__(template)

    def init_dict(self):
        # expect mapping for eop, opt and gml in namespace
        self.dict_info = {
            "id": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:identifier",
            "poslist": "gml:target/eop:Footprint/gml:multiExtentOf/gml:MultiSurface/gml:surfaceMembers/gml:Polygon/gml:exterior/gml:LinearRing/gml:posList",
            "beginposition": "gml:validTime/gml:TimePeriod/gml:beginPosition",
            "endposition": "gml:validTime/gml:TimePeriod/gml:endPosition",
            "parentidentifier": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:parentIdentifier",
            "status": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:status",
            "productType": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:productType",
            "acquisitionType": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:acquisitionType",
            "platformShortName": "gml:using/eop:EarthObservationEquipment/eop:platform/eop:Platform/eop:shortName",
            "platformserialid": "gml:using/eop:EarthObservationEquipment/eop:platform/eop:Platform/eop:serialIdentifier",
            "instrumentShortName": "gml:using/eop:EarthObservationEquipment/eop:instrument/eop:Instrument/eop:shortName",
            "cloudcover": "gml:resultOf/opt:EarthObservationResult/opt:cloudCoverPercentage",
            "snowcover": "gml:resultOf/opt:EarthObservationResult/opt:snowCoverPercentage",
            "ingestiondate": "gml:validTime/gml:TimePeriod/gml:endPosition",
            "productVersion": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:vendorSpecific/eop:SpecificInformation[eop:localAttribute='ProductVersion']/eop:localValue",
            "missingdatapercentage": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:vendorSpecific/eop:SpecificInformation[eop:localAttribute='MissingDataPercentage']/eop:localValue",
            "conformsto": "gml:resultOf/opt:EarthObservationResult/eop:product/eop:ProductInformation[1]/eop:referenceSystemIdentifier",
            "ql1": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[1]/eop:fileName",
            "epsg1": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[1]/eop:referenceSystemIdentifier",
            "ql2": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[2]/eop:fileName",
            "epsg2": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[2]/eop:referenceSystemIdentifier",
            "ql3": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[3]/eop:fileName",
            "epsg3": "gml:resultOf/opt:EarthObservationResult/eop:browse/eop:BrowseInformation[3]/eop:referenceSystemIdentifier",
            "archivingdate" : "gml:metaDataProperty/eop:EarthObservationMetaData/eop:archivedIn/eop:ArchivingInformation/eop:archivingDate",
            "overpassnumber" : "gml:metaDataProperty/eop:EarthObservationMetaData/eop:vendorSpecific/eop:SpecificInformation[eop:localAttribute='OverpassNumber']/eop:localValue",
            "cameraid" : "gml:metaDataProperty/eop:EarthObservationMetaData/eop:vendorSpecific/eop:SpecificInformation[eop:localAttribute='CameraID']/eop:localValue",
            "updatetime": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:archivedIn/eop:ArchivingInformation/eop:archivingDate",
            "processingdate": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:processing/eop:ProcessingInformation/eop:processingDate",
            "sensorType": "gml:using/eop:EarthObservationEquipment/eop:sensor/eop:Sensor/eop:sensorType",
            "resolution": "gml:using/eop:EarthObservationEquipment/eop:sensor/eop:Sensor/eop:resolution[@uom='m']",
            "compositeType": "gml:metaDataProperty/eop:EarthObservationMetaData/eop:processing/eop:ProcessingInformation/eop:compositeType",
            "orbitNumber": "gml:using/eop:EarthObservationEquipment/eop:acquisitionParameters/opt:Acquisition/eop:orbitNumber"
        }

    def fix_box(self, values, **kwargs):
        """  fixes the bounding box information from the posList value.
        note : lat long from hma is transformed to lon lat for geojson.
        """
        if values['poslist'] is not None:
            polygon = EsaPolygon().from_posList(pos_list=values['poslist'], is_lon_lat=False, **kwargs)
            values['bbox'] = polygon.get_bbox()
            values['geometry'] = polygon.as_geojson_string()

    def get_uid(self, values):
        """
        returns a generated uid for the probav products, based on the id.

        :param values: the key value association
        :type values: a dictionary
        :returns: the uid of the product
        :rtype: a String
        """
        last_index = values['id'].rfind(":")
        version = values['id'][last_index+1:]
        if "productVersion" not in values:
            values['productVersion'] = version
        tmp = values['id'][0:last_index]
        uid = tmp[tmp.rfind(":")+1:]+"_"+version
        return uid


class ProbavL2Parser(HmaParser):
    '''class that  able to parse a probav l2a HMA metadafile to
    create geojson equivalence.'''

    def __init__(self, template=None):
        """ Initialisator of the class
        """
        HmaParser.__init__(self, "PROBAV_L2.template" if template is None else template)

    def specific_treatment(self, product_file_info, values, xmltree=None):
        """ method that extracts specific information and insert them in the value list.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value association to update
        :type values: a dictionary
        :param xmltree: the xml tree representation of the metaile
        :type xmltree: an xml tree
        :returns: the values dictionary filled with specific values
        :rtype: a dictionary
        """

        uid = self.get_uid(values)
        values['title'] = uid
        self.fix_box(values)
        return uid


class ProbavSynthesisParser(HmaParser):
    '''class that able to parse a probav synthesis HMA metadafile to
    create geojson equivalence.'''

    def __init__(self, template=None):
        """ Initialisator of the class.
        """
        HmaParser.__init__(self, "PROBAV_S.template" if template is None else template)

    def get_before(self, name, tile):
        """ helper method that find the information present within the title definition.

        :param name: The name of the file.
        :type name: a String
        :param tile: the title reference
        :type tile: a String
        """
        pattern = "_([^_]+)_"+tile+"_.*"
        match = re.search(pattern, name)
        res = None
        if ((match) and (len(match.groups()) > 0)):
            res = match.group(1)
        return res

    def splitted_treatment(self, product_file_info, init_values, xmltree=None):
        """ method that extracts some specific information
        and allow the creation of several specific metadafiles.

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value aossication to update
        :type values: a dictionary
        """

        results = []
        loop_filesinfo = product_file_info.split(TileInfo.get_filter(TileInfo.XY))
        values = copy.deepcopy(init_values)
        for i in range(4):
            idxx = "epsg"+str(i)
            if (idxx in init_values) and (init_values[idxx] is not None):
                values[idxx] = init_values[idxx][init_values[idxx].rfind(":")+1:]
        values["conformsto"] = init_values["conformsto"][init_values["conformsto"].rfind(":")+1:]
        for key, filesnfo in loop_filesinfo.items():
            tile_ref = TileInfo.get(filesnfo.datas[0].name)
            values['tileid'] = str(tile_ref)
            bf_word = self.get_before(filesnfo.datas[0].name, values['tileid'])
            pos = init_values['id'].rfind(bf_word+"_")
            if pos > 0:
                pos = pos + len(bf_word) + 1
                values['id'] = init_values['id'][:pos] + values['tileid'] + "_" + init_values['id'][pos:]
            tile_ref.fix_box(values)
            uid = self.get_uid(values)
            values['title'] = uid

            self._set_titles(filesnfo)

            results.append(SpecificInfo(uid, filesnfo, copy.deepcopy(values)))
        return results

    def _set_titles(self, object_info):
        band_pattern = re.compile(r".*_([A-Z]+)\.tif")

        for info in list(object_info.datas):
            match = band_pattern.fullmatch(info.name)

            if match:
                info.title = match.group(1)


class ProbavV2Parser(HmaParser):
    """
    Parser for PROBA-V collection 2 product metadata.
    """
    def __init__(self, template=None):
        super().__init__("PROBAV_C2.template" if template is None else template)
        self.pattern_product_file_tiff = re.compile(r"PROBAV_.*_(?P<resolution>[A-Z0-9]+)_(?P<band>[A-Z]+)_V2\d\d\.tiff", re.IGNORECASE)
        self.pattern_product_tile = re.compile(r"PROBAV_S\d+_\w+_(?P<tile>X\d{2}Y\d{2})_\w+_V2\d{2}")

    def specific_treatment(self, product_file_info, values, xmltree=None):
        uid = self.get_uid(values)
        values['title'] = uid
        if "conformsto" in values:
            values["conformsto"] = values["conformsto"][values["conformsto"].rfind(":") + 1:]
        self.fix_box(values, datelineoffset=15)
        self.fix_file_info(product_file_info)

        if self.pattern_product_tile.match(values['title']):
            values['tileid'] = self.pattern_product_tile.match(values['title']).group('tile')

        return uid

    def get_uid(self, values):
        return values['id'][values['id'].rfind(":") + 1:]

    def fix_file_info(self, product_file_info: ObjectInfo):
        """
        Fix product file info.
        """
        for p in product_file_info.datas:
            if self.pattern_product_file_tiff.search(p.name):
                p.title = self.pattern_product_file_tiff.search(p.name).group('band')

        if len(product_file_info.datas) == 1 \
                and product_file_info.datas[0].title is None \
                and "NDVI" in product_file_info.datas[0].name:
            product_file_info.datas[0].title = "NDVI"


class SpotVgtParser(HmaParser):
    def __init__(self, template=None):
        super().__init__("SpotVGT_S.template" if template is None else template)
        self.pattern_product_group_id = re.compile(r"\w{9}_\d{8}_(?P<productType>[A-Z]*)_(?P<region>[A-Za-z\-]*)")
        self.pattern_product_title = re.compile(r".*_([A-Z0-9]*)\.\w*")
        self.pattern_quicklook = re.compile(r".*/SV\d{2}_VG\d_*")

    def specific_treatment(self, product_file_info, values, xmltree=None):
        uid = self.get_uid(values)
        values['title'] = uid
        values['orbitNumber'] = int(values['orbitNumber'])
        values["conformsto"] = values["conformsto"][values["conformsto"].rfind(":") + 1:]
        if values["parentidentifier"].split(":")[-1] == "VGT_S10":
            values['productGroupId'] = self.get_product_group_id(values)
        self.fix_box(values, datelineoffset=180, pole_wrap=True, fix_east_180=True)
        self.fix_products(product_file_info)
        return uid

    def get_product_group_id(self, values):
        if values["parentidentifier"].split(":")[-1] == "VGT_S10":
            id = values["id"].split(":")[-2]
            match = self.pattern_product_group_id.match(id)
            if match:
                return f"{match.group('region')}_{match.group('productType')}"
            else:
                return "GLOBAL_RADIO"
        else:
            return None

    def fix_products(self, product_file_info: ObjectInfo):
        for p in product_file_info.datas + product_file_info.related:
            if self.pattern_product_title.match(p.name):
                p.title = self.pattern_product_title.search(p.name).group(1)

        product_file_info.previews = [
            preview for preview in product_file_info.previews
            if self.pattern_quicklook.match(preview.name)
        ]
