"""
Created on 30 June 2019

@author: Tonneau Stephane
"""

import re
import logging

from oscars_mgmnt.parser.base import XMLMetaParser

INSPIRE_LOGGER = logging.getLogger('InspireParser')


class InspireParser(XMLMetaParser):
    """ Generic parser used to transform INSPIRE metadata files to GeoJSON."""

    def __init__(self, template=None):
        """ Constructor of the class
        """
        super().__init__(template)

    def init_dict(self):
        # expect mapping for gmd, gco and gml in namespace
        self.dict_info = {
            "id": "gmd:fileIdentifier/gco:CharacterString",
            "westbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal",
            "southbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal",
            "eastbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal",
            "northbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal",
            "beginposition": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition",
            "endposition": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:endPosition",
            "parentidentifier": "gmd:parentIdentifier/gco:CharacterString",
            # "status": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:status/gmd:MD_ProgressCode", # not used?
            "title": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString",
            "conformsto": "gmd:referenceSystemInfo/gmd:MD_ReferenceSystem/gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:code/gco:CharacterString",
            "vito_dataquality": "gmd:dataQualityInfo/gmd:DQ_DataQuality/gmd:lineage/gmd:LI_Lineage/gmd:source/gmd:LI_Source/gmd:sourceCitation/gmd:CI_Citation/gmd:title/gco:CharacterString",
            "updatetime": "gmd:dateStamp/gco:DateTime"
        }

    def extract_info(self, xmltree, dict_values, dict_extract):
        for key, val in dict_extract.items():
            result = xmltree.findall(val, self.namespace)
            if result is not None and len(result) > 0:
                # check if the first hit is not empty or None
                if result[0].text:
                    dict_values[key] = str(result[0].text)


class CGSParser(InspireParser):
    """ Parser used to transform CGS INSPIRE metadata files to GeoJSON.."""

    def __init__(self, template=None):
        """ Constructor of the class
        """
        InspireParser.__init__(self, "CGS_S2.template" if template is None else template)

    def fix_products(self, fileobject):
        """ helper method that moves files present in "datas" to "related".
        :param fileobject: the files associated to a product
        :type fileobject: a FileObject
        """
        raise NotImplementedError("Implement this method in a subclass.")

    def specific_treatment(self, product_file_info, values, xmltree=None):
        """ method that extracts some specific information and insert them in "values".

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value association to update
        :type values: a dictionary
        :param xmltree: the xml tree representation of the metaile
        :type xmltree: an xml tree
        :returns: the values dictionary filled with specific values
        :rtype: a dictionary
        """
        uid = values['id'][values['id'].rfind(":") + 1:]

        if "_V" in uid:
            values['productVersion'] = uid[uid.rfind("_V") + 1:]

        values['platformserialid'] = uid.split("_")[0]

        self.fix_products(product_file_info)

        return uid

    @staticmethod
    def format_uid_date(uid_date):
        return '{}-{}-{}T{}:{}:{}Z'.format(uid_date[0:4], uid_date[4:6], uid_date[6:8],
                                           uid_date[9:11], uid_date[11:13], uid_date[13:15])


class CGS_S1_Parser(CGSParser):

    def __init__(self, template=None):
        super().__init__('CGS_S1.template' if template is None else template)

    def init_dict(self):
        super().init_dict()

        self.dict_info.update(
            {
                "updatetime": "gmd:dateStamp/gco:Date",
                "resolution": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:spatialResolution/gmd:MD_Resolution/gmd:equivalentScale/gmd:MD_RepresentativeFraction/gmd:denominator/gco:Integer",
                "poslist": ".//gml:Polygon/gml:exterior/gml:LinearRing/gml:posList",
            }
        )

    def specific_treatment(self, product_file_info, values, xmltree=None):
        uid = super().specific_treatment(product_file_info, values, xmltree)
        INSPIRE_LOGGER.info("Processing %s", uid)

        uid_split = uid.split("_")

        # set producttype
        # eg. CGS_S1_GRD_L1 --> GRD
        producttype = values["parentidentifier"].split("_")[-2]
        values["producttype"] = producttype
        if producttype in ['SLC', 'SIGMA0']:
            uid_date = uid_split[5]
        else:  # GRD and COHERENCE
            uid_date = uid_split[4]
        values["nominaldatetime"] = self.format_uid_date(uid_date)

        # fix updatetime property, create timestamp from date
        values["updatetime"] = values["updatetime"] + "T00:00:00Z"

        if values['poslist'] is not None:
            from oscars_mgmnt.geometry.polygon import EsaPolygon
            polygon = EsaPolygon().from_posList(pos_list=values['poslist'], is_lon_lat=False, simplify=False)
            values['geometry'] = polygon.as_geojson_string()

        if values["producttype"] in ["SIGMA0", "COHERENCE"]:
            extra_lookup = {
                "resolution": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:spatialResolution/gmd:MD_Resolution/gmd:distance/gco:Distance",
                "sourceproduct": "gmd:dataQualityInfo/gmd:DQ_DataQuality/gmd:lineage/gmd:LI_Lineage/gmd:source/gmd:LI_Source/gmd:sourceCitation/gmd:CI_Citation/gmd:title/gco:CharacterString",
                "operation": "gmi:acquisitionInformation/gmi:MI_AcquisitionInformation/gmi:operation/gmi:MI_Operation/gmi:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString"
            }

            self.extract_info(xmltree, values, extra_lookup)

            if values["producttype"] == "SIGMA0":
                if "sourceproduct" in values:
                    # preferred way of extracting the orbitnumber
                    values["orbitnumber"] = int(values["sourceproduct"].split("_")[-3])
                    values["relativeorbitnumber"] = self.calculate_relative_orbit_number(values["platformserialid"],
                                                                                         values["orbitnumber"])
                elif "operation" in values:
                    # use operation field in case the source product isn't mentioned
                    values["orbitnumber"] = int(values["operation"].split("_")[2].split(".")[0])
                    values["relativeorbitnumber"] = self.calculate_relative_orbit_number(values["platformserialid"],
                                                                                         values["orbitnumber"])

                polarisationAbbr = uid[19:21]
                values["sensormode"] = uid.split("_")[1]
                values["orbitdirection"] = uid.split("_")[6]

            elif values["producttype"] == "COHERENCE":
                # absolute orbit number doesn't apply for coherence products, as it combines two products
                values["relativeorbitnumber"] = uid_split[-2]
                values["orbitdirection"] = "ASCENDING" if uid_split[-3] == "ASC" else "DESCENDING"

                # don't include platformserialid for coherence products
                values.pop('platformserialid', None)

                if "sourceproduct" in values:
                    sourceproduct_split = list(filter(lambda s: len(s) > 0, values["sourceproduct"].split("_")))
                    values["sensormode"] = sourceproduct_split[1]
                    polarisationAbbr = sourceproduct_split[3][2:]
                else:
                    # put default values
                    values["sensormode"] = "IW"
                    polarisationAbbr = "DV"

        elif values["producttype"] in ["GRD", "SLC"]:
            values["orbitnumber"] = int(uid.split("_")[-3])
            values["relativeorbitnumber"] = self.calculate_relative_orbit_number(values["platformserialid"],
                                                                                 values["orbitnumber"])
            polarisationAbbr = uid[14:16]
            values["sensormode"] = uid.split("_")[1]

        polarisation = {
            "SH": {"mode": "S", "channels": "HH"},
            "SV": {"mode": "S", "channels": "VV"},
            "DH": {"mode": "D", "channels": "HH, HV"},
            "DV": {"mode": "D", "channels": "VV, VH"},
            "HH": {"mode": "D", "channels": "HH"},
            "HV": {"mode": "D", "channels": "HV"},
            "VV": {"mode": "D", "channels": "VV"},
            "VH": {"mode": "D", "channels": "VH"}
        }
        values["polarisationmode"] = polarisation[polarisationAbbr]["mode"]
        values["polarisationchannels"] = polarisation[polarisationAbbr]["channels"]

        return uid

    def fix_products(self, fileobject):
        grd_pattern = re.compile(r".*S1._.._GRDH_.*.zip")
        slc_pattern = re.compile(r".*S1._.._SLC__.*.zip")
        sigma0_pattern = re.compile(r".*S1._.._GRDH_SIGMA0_.*_V\d{3}_(angle|VV|VH).tif")
        coherence_pattern = re.compile(r".*S1._S1._Coherence_.*_V\d{3}_(VV|VH).tif")

        for info in list(fileobject.datas):
            if sigma0_pattern.match(info.name):
                info.title = sigma0_pattern.search(info.name).group(1)
            elif coherence_pattern.match(info.name):
                info.title = coherence_pattern.search(info.name).group(1)
            elif grd_pattern.match(info.name):
                info.title = "GRD"
            elif slc_pattern.match(info.name):
                info.title = "SLC"

    @staticmethod
    def calculate_relative_orbit_number(platformserialid: str, orbitnumber: int):
        # based on https://forum.step.esa.int/t/sentinel-1-relative-orbit-from-filename/7042
        if platformserialid == "S1A":
            return ((orbitnumber - 73) % 175) + 1
        elif platformserialid == "S1B":
            return ((orbitnumber - 27) % 175) + 1


class CGS_S2_Parser(CGSParser):
    def __init__(self, template=None):
        super().__init__('CGS_S2.template' if template is None else template)

    def init_dict(self):
        super().init_dict()

        self.dict_info.update(
            {
                "platformShortName": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:aggregationInfo[2]/gmd:MD_AggregateInformation/gmd:initiativeType/gmd:DS_InitiativeTypeCode",
                "instrumentShortName": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:aggregationInfo[3]/gmd:MD_AggregateInformation/gmd:initiativeType/gmd:DS_InitiativeTypeCode",
                "cloudcover": "gmd:contentInfo/gmd:MD_ImageDescription/gmd:cloudCoverPercentage/gco:Real",
                "missingdatapercentage": "gmd:dataQualityInfo/gmd:DQ_DataQuality/gmd:report/gmd:DQ_CompletenessOmission/gmd:result/gmd:DQ_QuantitativeResult/gmd:value/gco:Record"
            }
        )

    def specific_treatment(self, product_file_info, values, xmltree=None):
        uid = super().specific_treatment(product_file_info, values, xmltree)

        producttype = values["parentidentifier"].split("_")[-2]
        values["producttype"] = producttype

        if 'vito_dataquality' in values and len(values['vito_dataquality']) > 5:
            pattern = r"[^a-zA-Z0-9]\d{8,8}T\d{6,6}[^a-zA-Z0-9][A](\d{6,6})[^a-zA-Z0-9]"
            match = re.search(pattern, values['vito_dataquality'])
            if ((match) and (len(match.groups()) >= 1)):
                res = match.group(1)
                # values['orbitdirection'] = "ASCENDING"
                values['orbitnumber'] = int(res)

        if "gmi" in self.namespace:
            extra_lookup = {
                "operation": "gmi:acquisitionInformation/gmi:MI_AcquisitionInformation/gmi:operation/gmi:MI_Operation/gmi:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString"
            }
            self.extract_info(xmltree, values, extra_lookup)
            pattern = r"^(\d+)_\d+_((?:ASCENDING)|(?:DESCENDING))$"
            match = re.search(pattern, values['operation'])
            if match and len(match.groups()) >= 1:
                values['relativeorbitnumber'] = int(match.group(1))
                values['orbitdirection'] = match.group(2)

        if "updatetime" not in values:
            self.extract_info(xmltree, values, {"updatetime": "gmd:dateStamp/gco:Date"})
            if "updatetime" in values:
                # fix updatetime property, create timestamp from date
                values["updatetime"] = values["updatetime"] + "T00:00:00Z"

        uid_split = uid.split("_")
        if all(s in uid for s in ("S2", "L1C")):
            values['tileid'] = uid_split[5][1:]
            uid_date = uid_split[2]
            values['nominaldatetime'] = self.format_uid_date(uid_date)
        else:
            values['tileid'] = uid_split[2]
            uid_date = uid_split[1]
            values['nominaldatetime'] = self.format_uid_date(uid_date)

        res_pattern = re.compile(r'.*(_\d{2,3}M_).*')  # e.g. _10M_, _100M_
        if res_pattern.match(uid):
            values['resolution'] = res_pattern.search(uid).group(1)[1:-2]

        return uid

    def fix_products(self, fileobject):
        """ helper method that moves files present in "datas" to "related".
        :param fileobject: the files associated to a product
        :type fileobject: a FileObject
        """

        # the ?: in front makes it non-capturing (so it doesn't introduce and additional group)
        shadow_pattern = re.compile(
            r'.*((?:SHADOWMASK_\d{2}M)|RAA_\d{2}M|VZA_\d{2}M|SZA_\d{2}M|WVP_\d{2}M|AOT_\d{2}M).*')
        cloud_pattern = re.compile(r'.*(CLOUDMASK_\d{2}M).*')
        scene_pattern = re.compile(r'.*((?:SCENECLASSIFICATION_|PIXELCLASSIFICATION_)\d{2}M).*')
        biopar_pattern = re.compile(r'.*((?:FAPAR|NDVI|FCOVER|LAI|CCC|CWC)_\d{2}M).*')
        water_pattern = re.compile(r'.*((?:CHL|CHL_SOURCE|SPM|TUR|RHOW)_\d{2}M).*')
        water_rhow_pattern = re.compile(r'.*RHOW-((?:B\d\d|B8A))_\d{2}M.*')
        radiometry_pattern = re.compile(r'.*((?:TOC-B\d\d_\d{2}M|TOC-B8A_20M)).*')
        L1C_pattern = re.compile(r'.*_((?:TCI|B\d{2}|B8A))\..*')

        for info in list(fileobject.datas):

            if shadow_pattern.match(info.name):
                info.title = shadow_pattern.search(info.name).group(1)
                info.category = "QUALITY"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif cloud_pattern.match(info.name):
                info.title = cloud_pattern.search(info.name).group(1)
                info.category = "CLOUD"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif scene_pattern.match(info.name):
                info.title = scene_pattern.search(info.name).group(1)
                info.category = "QUALITY"
                fileobject.datas.remove(info)
                fileobject.related.append(info)
            elif biopar_pattern.match(info.name):
                info.title = biopar_pattern.search(info.name).group(1)
            elif water_pattern.match(info.name):
                info.title = water_pattern.search(info.name).group(1)
            elif water_rhow_pattern.match(info.name):
                info.title = water_rhow_pattern.search(info.name).group(1)
            elif radiometry_pattern.match(info.name):
                info.title = radiometry_pattern.search(info.name).group(1)
            elif L1C_pattern.match(info.name):
                info.title = L1C_pattern.search(info.name).group(1)


class DEMParser(XMLMetaParser):
    """ Parser used to transform DEM INSPIRE metadata files to GeoJSON."""

    def __init__(self, template=None):
        """ Constructor of the class
        """
        super().__init__('DEM.template' if template is None else template)

    def init_dict(self):
        # expect mapping for gmd, gco and gml in namespace
        self.dict_info = {
            "westbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal",
            "southbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal",
            "eastbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal",
            "northbbox": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal",
            "poslist": ".//gml:Polygon/gml:exterior/gml:LinearRing/gml:posList",
            "beginposition": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition",
            "endposition": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:endPosition",
            "updatetime": "gmd:dateStamp/gco:Date",
            "resolution": "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:spatialResolution/gmd:MD_Resolution/gmd:distance/gco:Distance"
        }

    def specific_treatment(self, product_file_info, values, xmltree=None):
        """ method that extracts some specific information and insert them in "values".

        :param product_file_info: the description of the files associated to a product
        :type product_file_info: a file object info
        :param values: the key value association to update
        :type values: a dictionary
        :param xmltree: the xml tree representation of the metaile
        :type xmltree: an xml tree
        :returns: the values dictionary filled with specific values
        :rtype: a dictionary
        """
        # Properties
        title = product_file_info.datas[0].name.split("/")[-1].split(".tif")[0]
        values['title'] = title
        if float(values['resolution']) == 1.0:
            values['parentidentifier'] = "urn:eop:VITO:COP_DEM_GLO_30M_COG"
        else:
            values['parentidentifier'] = "urn:eop:VITO:COP_DEM_GLO_90M_COG"
        values['id'] = values['parentidentifier'] + ":" + title
        values['updatetime'] = values['updatetime'] + "T00:00:00Z"
        values['beginposition'] = values['beginposition'] + "Z"
        values['endposition'] = values['endposition'] + "Z"

        # Geometry
        if values['poslist'] is not None:
            from oscars_mgmnt.geometry.polygon import EsaPolygon
            polygon = EsaPolygon().from_posList(pos_list=values['poslist'], is_lon_lat=False, simplify=False)
            values['geometry'] = polygon.as_geojson_string()

        # AcquisitionInformation
        tile_info = title.split("Copernicus_DSM_COG_")[1].split("_")[1:]
        values['tileId'] = tile_info[0] + tile_info[2]

        # AdditionalAttributes
        values['resolution'] = str(float(values['resolution']) * 30)
        reference_system_info_path = "gmd:referenceSystemInfo/gmd:MD_ReferenceSystem/" \
                                     "gmd:referenceSystemIdentifier/gmd:RS_Identifier/" \
                                     "gmd:code/gco:CharacterString"
        reference_system_info = xmltree.findall(reference_system_info_path, self.namespace)
        if reference_system_info is not None and len(reference_system_info) == 2:
            horizontal_text = reference_system_info[0].text
            vertical_text = reference_system_info[1].text
            if "horizontal: " in horizontal_text:
                values['horizontalReferenceId'] = horizontal_text.split("horizontal: ")[1]
                if values['horizontalReferenceId'] == "https://epsg.io/4326":
                    values['horizontalReferenceId'] = "https://www.opengis.net/def/crs/EPSG/0/4326"
                values['conformsto'] = values['horizontalReferenceId']
            if "vertical:" in vertical_text:
                values['verticalReferenceId'] = vertical_text.split("vertical:")[1]
                if values['verticalReferenceId'] == "https://epsg.io/3855":
                    values['verticalReferenceId'] = "https://www.opengis.net/def/crs/EPSG/0/3855"

        # ProductInformation
        values['producttype'] = "DEM_GLO_30M"
        if values['resolution'] == "90":
            values['producttype'] = "DEM_GLO_90M"

        uid = super().specific_treatment(product_file_info, values, xmltree)
        return uid
