import json
import subprocess
import logging

import shapely
import shapely.geometry

from oscars_mgmnt.parser.base import BaseParser
from oscars_mgmnt.file_object import ObjectInfo

LOGGER = logging.getLogger(__name__)

class EuDemParser(BaseParser):
    def __init__(self):
        super().__init__(template='EU_DEM.template')

    def convert(self, product_file_info):

        import os
        import time

        # Initialize values dict
        values = {}

        # filename without extension
        file_name = os.path.basename(product_file_info.get_product_path()).split('.')[0]
        file_name_parts = file_name.split('_')
        #eu_dem_v11_E00N20
        tile_id = file_name_parts[3]

        version = file_name_parts[2] # v100 or V200, ...
        major_version = int(version[1])

        values['producttype'] = 'DEM_EU_25M'
        parent_identifier= 'urn:eop:VITO:COP_DEM_EU_25M'
        title = file_name

        #Going to retrieve bounding box and polygon for file
        input_file = product_file_info.get_product_path() + '/' + file_name + '.TIF'
        LOGGER.debug('Going to get polygon & bounding box')
        command = ["/usr/bin/gdalinfo", input_file, '-json']
        print(command)
        result = subprocess.run(command, check=True, stdout=subprocess.PIPE)
        output = result.stdout
        json_output = json.loads(output)
        wgsExtent = json_output['wgs84Extent']
        polygon = shapely.geometry.shape(wgsExtent)
        bbox = polygon.bounds

        values['id'] = parent_identifier + ':' + title
        values['title'] = title
        values['parentidentifier'] = parent_identifier
        values['conformsto'] = 'https://www.opengis.net/def/crs/EPSG/0/3035'
        values['horizontalReferenceId'] = 'https://www.opengis.net/def/crs/EPSG/0/3035'
        values['verticalReferenceId'] = 'https://www.opengis.net/def/crs/EPSG/0/4258'
        values['westbbox'] = bbox[0]
        values['southbbox'] = bbox[1]
        values['eastbbox'] = bbox[2]
        values['northbbox'] = bbox[3]
        values['tileid'] = tile_id
        values['resolution'] = 25

        values['date'] = '2011-01-01T00:00:00Z'
        values['beginposition'] = '2011-01-01T00:00:00Z'
        values['endposition'] = '2011-12-31T23:59:59Z'
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['updatetime'] = time.strftime(
            '%Y-%m-%dT%H:%M:%SZ',
            product_file_info.storage.get_time_modified(product_file_info.get_product_path())
        )

        if self.jinja_template is None:
            self.with_template(self.template)

        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):

        for file_info_data in list(product_file_info.datas):
            if file_info_data.name.endswith('.TIF'):
                file_info_data.title = 'DEM'

        self.fix_products(product_file_info)
        return values['id']

    def fix_products(self, fileobject):
        # eu_dem_v11_E00N20.TIF.ovr
        for info in list(fileobject.datas):
            if info.name.endswith('.ovr'):
                info.type = "image/tiff"
                fileobject.datas.remove(info)
                fileobject.related.append(info)

        for info in list(fileobject.related):
            if info.name.endswith('.TFw'):
                info.type= "text/TFw"
