"""
Created on 30 June 2019

@author: Tonneau Stephane
"""

import os
import logging
from oscars_mgmnt.utils.common import OutputWriter, Jsonify, JsonValidator

FILEWRITER_LOGGER = logging.getLogger('FileWriter')


class FileWriter(OutputWriter):
    '''Writer that saves metadata documents on the file system'''

    def __init__(self, dest):
        """ Initialisator of the class.

        :param dest: the specific directory where files must be written
        :type dest: a string
        """
        OutputWriter.__init__(self)
        self.target_dir = dest
        if (self.target_dir is not None) and not(os.path.exists(self.target_dir)):
            os.mkdir(self.target_dir)

    def output(self, uid, content):
        """ writes the content (translated into geojson) in a file,
        whose filename is based on the uid.

        :param uid: a uniq reference
        :type uid: a string
        :param content: a string containing the metadata content to save.
        :type content: a string
        """
        the_output = os.path.join(self.target_dir, uid + '.json')
        with open(the_output, 'w') as fh:
            fh.write(content)
        FILEWRITER_LOGGER.info("Metafile %s : written as %s", str(uid), the_output)


class GeojsonWriter(FileWriter):
    '''Writer that saves metadata documents as geojson on the file system.'''

    def __init__(self, dest, schema=None, pure=False):
        """ Initialisator of the class

        :param dest: the specific directory where files must be written
        :type dest: a string
        :param schema: the path to a the schema that must be used to validate the file.
        :type schema: a string
        :type pure: indicates if conversion to pure geojson must be realized or not.
        :type pure: a boolean
        """
        FileWriter.__init__(self, dest)
        self.validator = None if schema is None else JsonValidator(schema)
        self.pure = pure

    def output(self, uid, content):
        """ writes the content (translated into geojson) in a file,
        whose filename is based on the uid.

        :param uid: a uniq reference
        :type uid: a string
        :param content: a string containing the metadata content to save.
        :type content: a string
        """
        try:
            json_message = Jsonify(content, self.validator)
            json_result = json_message.to_geoJson()
        except:
            the_output = os.path.join(self.target_dir, uid + '.error')
            with open(the_output, 'w') as fh:
                fh.write(str(content))
            raise
        if self.target_dir is not None:
            the_output = os.path.join(self.target_dir, uid + '.geojson')
            with open(the_output, 'w') as fh:
                fh.write(json_result if self.pure else str(json_message))
            FILEWRITER_LOGGER.info("Metafile %s : written as geojson %s", str(uid), the_output)
