import itertools
import logging
from json.decoder import JSONDecodeError

import requests
import requests.adapters

LOGGER = logging.getLogger(__name__)


class ElasticRestSupport:
    """ Class that provides support for retrieving Elasticsearch documents over REST calls """

    def __init__(self, hosts: list):
        self.hosts = itertools.cycle(hosts if hosts is not None else [])
        self.session = requests.Session()

        retry = requests.adapters.Retry()
        self.session.mount("http://", requests.adapters.HTTPAdapter(max_retries=retry))
        self.session.mount("https://", requests.adapters.HTTPAdapter(max_retries=retry))

    def get(self, index, identifier):
        """
        Get the source of an Elasticsearch document by the identifier
        :param index: Elasticsearch index
        :type index: str
        :param identifier: document identifier
        :type identifier: str
        :rtype: dict
        """
        return self._get_with_retry(index, identifier)

    def _get_with_retry(self, index: str, identifier: str, max_retries: int = 3, timeout=60):
        for num_try in range(max_retries):
            try:
                response = self.session.get(
                    f"http://{next(self.hosts)}:9200/{index}/_source/{identifier}", timeout=timeout
                )
                if response.status_code == requests.codes.ok:
                    return response.json()
                elif response.status_code == requests.codes.not_found:
                    LOGGER.error(f"Product with identifier {identifier} was not found in Elasticsearch")
            except requests.exceptions.RequestException as e:
                LOGGER.error("Error while sending request to retrieve product metadata from Elasticsearch")
            except JSONDecodeError as e:
                LOGGER.error("Could not decode Elasticsearch response to JSON")

        raise Exception("Failed to get product metadata from Elasticsearch")