from oscars_mgmnt.utils.common import Config
import glob
import fnmatch
import os
import time
import logging
from typing import Iterator, List

LOGGER = logging.getLogger(__name__)


class StorageInput:

    def __init__(self):
        pass

    def glob(self, path: str) -> Iterator[str]:
        """
        Get the paths matching the provided glob pattern.

        :param path: glob path pattern
        :returns: iterator of paths
        """
        raise NotImplementedError

    def list_files_recursively(self, path: str) -> Iterator[str]:
        """
        List all files in the given directory path recursively. If the path is a file, only that file will be returned.

        :param path: path of file or directory to list
        :returns: iterator of file paths
        """
        raise NotImplementedError

    def get_filesize(self, path: str) -> int:
        """
        Get the file size, in bytes, of the given path.

        :param path: path of the file
        :returns: file size in bytes
        """
        raise NotImplementedError

    def get_time_modified(self, path) -> time.struct_time:
        """
        Get the modification timestamp of the given file path.

        :param path: path of the file
        :returns: modification date of file
        """
        raise NotImplementedError

    def open_file_lines(self, path: str):
        raise NotImplementedError


class FileSystemInput(StorageInput):

    def glob(self, path: str) -> Iterator[str]:
        return glob.iglob(path)

    def list_files_recursively(self, path: str) -> Iterator[str]:
        if os.path.isfile(path):
            yield path
        elif os.path.isdir(path):
            for filename in glob.iglob(os.path.join(path, "**"), recursive=True):
                if os.path.isfile(filename):
                    yield filename

    def get_filesize(self, path: str) -> int:
        return os.path.getsize(path)

    def get_time_modified(self, path) -> time.struct_time:
        return time.localtime(os.path.getmtime(path))

    def open_file_lines(self, path: str):
        return open(path, 'r')


class S3Input(StorageInput):

    s3_uri_template = "s3://{bucket}/{key}"

    def __init__(self, config: Config):
        super().__init__()
        LOGGER.debug("Initialising S3 client")

        import boto3
        import botocore.session
        import botocore.client

        botocore_session = botocore.session.Session()
        botocore_session.unregister('before-parameter-build.s3', botocore.handlers.validate_bucket_name)
        boto3.setup_default_session(botocore_session=botocore_session)

        if not config.s3_endpoint_url:
            raise Exception("Please provide S3 details in the configuration")

        # S3 credentials can be passed through environment variables
        # AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY
        self.s3 = boto3.resource(
            's3',
            endpoint_url=config.s3_endpoint_url,
            config=botocore.client.Config(signature_version=botocore.UNSIGNED) if config.s3_unsigned else None
        )

    def get_buckets(self):
        return self.s3.buckets.all()

    def get_bucket_by_name(self, name: str):
        return self.s3.Bucket(name)

    def get_objects(self, bucket, prefix: str = None):
        if isinstance(bucket, str):
            bucket = self.get_bucket_by_name(bucket)

        if prefix:
            return bucket.objects.filter(Prefix=prefix)
        else:
            return bucket.objects.all()

    @staticmethod
    def split_bucket_key(uri: str):
        _, tmp = uri.split("://", 1)
        bucket, key = tmp.split("/", 1)
        return bucket, key

    @staticmethod
    def contains_glob(path: str) -> bool:
        return any(char in path for char in ["*", "?", "["])

    @staticmethod
    def matches_glob(path: str, glob: str) -> bool:
        """
        Checks if the path matches the glob pattern. The path segments are evaluated separately to their corresponding
        glob segment. Only if all parts match their glob segment, true will be returned.

        :param path: path to evaluate
        :param glob: glob pattern
        :returns: true if the path matches the glob pattern
        """
        path_parts = path.split("/")
        glob_parts = glob.split("/")

        if len(path_parts) != len(glob_parts):
            return False

        return all(fnmatch.fnmatch(ipath, iglob) for ipath, iglob in zip(path_parts, glob_parts))

    def get_object_from_uri(self, uri: str):
        bucket_name, key = self.split_bucket_key(uri)
        bucket = self.get_bucket_by_name(bucket_name)
        return bucket.Object(key)

    def get_filesize(self, path: str) -> int:
        obj = self.get_object_from_uri(path)
        return obj.content_length

    def get_time_modified(self, path) -> time.struct_time:
        obj = self.get_object_from_uri(path)
        return obj.last_modified.timetuple()

    def list_files_recursively(self, path: str) -> Iterator[str]:
        bucket_name, key = self.split_bucket_key(path)
        bucket = self.get_bucket_by_name(bucket_name)
        objects = self.get_objects(bucket, prefix=key)
        for obj in objects:
            yield self.s3_uri_template.format(bucket=bucket.name, key=obj.key)

    def glob(self, path: str):
        bucket_glob, key_glob = self.split_bucket_key(path)
        if self.contains_glob(bucket_glob):
            # get all buckets and filter those that match the glob pattern
            buckets = {
                bucket
                for bucket in self.get_buckets()
                if S3Input.matches_glob(bucket.name, bucket_glob)
            }
        else:
            buckets = {self.get_bucket_by_name(bucket_glob)}

        for bucket in buckets:
            # get the objects in the bucket
            LOGGER.debug(f"Checking objects in bucket {bucket.name}")
            for key in self.walk(bucket, key_glob):
                yield self.s3_uri_template.format(bucket=bucket.name, key=key)

    @staticmethod
    def walk(bucket, path: str, prefix: str = '', delimiter: str = '/',
             list_files: bool = True, list_dirs: bool = True):
        """
        Iterator that lists a bucket's files and directories matching a given glob path.
        This function is called recursively to deplete the path segments and build up the prefix.

        :param bucket: bucket to list content for
        :param path: glob path pattern
        :param prefix: path prefix (optional)
        :param delimiter: directory delimiter (optional)
        :param list_files: list files in result (optional)
        :param list_dirs: list directories in result (optional)
        :returns: iterator of keys
        """
        path = path.rstrip('/')
        final_depth = len(path.split("/")) == 1
        LOGGER.debug(f"Walking bucket {bucket.name} with prefix '{prefix}' and path '{path}'")

        paginator = bucket.meta.client.get_paginator('list_objects')
        for resp in paginator.paginate(Bucket=bucket.name, Prefix=prefix, Delimiter=delimiter):
            tmp: List[str] = []
            if 'CommonPrefixes' in resp and (list_dirs or not final_depth):
                dirs = {
                    f['Prefix']
                    for f in resp['CommonPrefixes']
                    if S3Input.matches_glob(f['Prefix'].split('/')[-2], path.split('/')[0])
                }
                tmp += dirs
            if 'Contents' in resp and final_depth and list_files:
                files = {
                    f['Key']
                    for f in resp['Contents']
                    if S3Input.matches_glob(f['Key'].split('/')[-1], path.split('/')[0])
                }
                tmp += files

            if final_depth:
                for p in tmp:
                    yield p
            else:
                for p in tmp:
                    for i in S3Input.walk(bucket, os.path.join(*path.split('/')[1:]), prefix=p, delimiter=delimiter,
                                          list_files=list_files, list_dirs=list_dirs):
                        yield i

    def open_file_lines(self, path: str):
        obj = self.get_object_from_uri(path)
        return obj.get()['Body'].iter_lines()
